# Clique Percolation blog post
# Eiko Fried, psych-networks.com
# Last updated: November 5th 2019

# Based on and with quotes from: https://cran.r-project.org/web/packages/CliquePercolation/vignettes/CliquePercolation.html



### load libraries
library('qgraph')
library('CliquePercolation')
library('bootnet')
library('Matrix')
library('EGAnet')

### load data
setwd("~/Dropbox/websites/psych-networks.com/2019-11 Clique Percolation")
load('data.Rdata')



### 1. Estimate network & use DSM-5 communities
n1 <- estimateNetwork(data, default="EBICglasso")

### plot network
names<-c('B1','B2', 'B3', 'B4', 'B5', 'C1', 'C2', 'D1', 'D2', 'D3', 'D4', 'D5', 'D6', 'D7', 'E1', 'E2', 'E3', 'E4', 'E5', 'E6')
longnames <- c('Intrusive thoughts', 'Nightmares', 'Flashbacks', 'Emotional cue reactivity', 'Physiological cue reactivity', 'Avoidance of thoughts', 'Avoidance of reminders', 'Trauma-related amnesia', 'Negative beliefs', 'Blame of self or others', 'Negative trauma-related emotions', 'Loss of interest', 'Detachment', 'Restricted affect', 'Irritability/anger', 'Self-destructive/reckless behavior', 'Hypervigilance', 'Exaggerated startle response', 'Difficulty concentrating', 'Sleep disturbance')
gr1 <- list('Intrusions'=c(1:5), 'Avoidance'=c(6:7), 'Cognition & mood alterations'=c(8:14),'Arousal & reactivity alterations'=c(15:20)) 

pdf("Network1.pdf", width=8.5, height=5)
g1<-plot(n1, labels=names, layout="spring", vsize=6, cut=0, border.width=1.5, border.color='black', title="DSM-5 communities",
         groups=gr1, color=c('#a8e6cf', '#dcedc1', '#ffd3b6', '#ff8b94'), nodeNames = longnames,legend.cex=.35)
dev.off()

L <- g1$layout



### 2. Traditional community detection (walktrap)
comm1<- EGA(data, plot.EGA = TRUE); comm1
gr2 <- list('Intrusions & Avoidance'=c(1:7), 'Cogn. & mood alterations, arousal'=c(8,9,12:20), 'Blame/emotions'=c(10:11)) 

pdf("Network2.pdf", width=8.5, height=5)
g2<-plot(n1, layout=L, labels=names, vsize=6, cut=0, border.width=1.5, border.color='black', title="Walktrap communities",
          groups=gr2, color=c('#a8e6cf', '#ff8b94', '#ffd3b6'), nodeNames = longnames,legend.cex=.35)
dev.off()



### 3. Clique Percolation by optimizing k and I
W <- qgraph(n1$graph) 

# To run the clique percolation algorithm for weighted networks, we initially need to optimize k and I. 
# I = intensity of a community; needs to exceed a certain threshold to be extracted as community
# k = minimum clique size in terms of nodes
# For each k & I combination, we determine the threshold and save k, I, # of communities, and other details
# NAs in output because ratio threshold requires ≥2, χ ≥3 communities

thresholds <- cpThreshold(W, method = "weighted", k.range = 3,       # we search for cliques with at least 3 nodes, because we are not allowed by the program to search for k=2 cliques
                          I.range = c(seq(0.20, 0.01, by = -0.01)), # we search for I values in steps of 0.05 between 0.01 and 0.20
                          threshold = c("largest.components.ratio","chi")); thresholds

# To pick optimal I values for each k, we check at which I the ratio threshold jumps to a high level & crosses 2
# I just before jump should ideally also have highest χ
# Looking at 'thresholds', we therefore pick k=3, I=.12, where ratio exceeds 2 for the first time (2.167) with a very large χ

results <- cpAlgorithm(W, k = 3, method = "weighted", I = 0.12)
results$list.of.communities.numbers

pdf("Network3.pdf", width=8.5, height=5)
g3 <- cpColoredGraph(W, list.of.communities = results$list.of.communities.numbers, layout=L, theme='colorblind',
                     labels=names, vsize=6, cut=0, border.width=1.5, 
                     border.color='black', nodeNames = longnames,legend.cex=.37,
                     edge.width = 1, title ="Clique Percolation (optimizing I & k)")
dev.off()



### 4. use Clique Percolation by optimizing entropy (based on Shannon information)

# the idea here is to identify the most 'surprising' community partition ... 
# ... defined as low probability of knowing to which community a randomly picked node belongs 

thresholds2 <- cpThreshold(W, method = "weighted", k.range = 3,     
                          I.range = c(seq(0.20, 0.01, by = -0.01)), 
                          threshold = "entropy"); thresholds2

set.seed(1337)
permute <- cpPermuteEntropy(W, cpThreshold.object = thresholds2,
                            n = 100, interval = 0.95)
permute$Confidence.Interval     # CI values of entropy
permute$Extracted.Rows          # relevant final solutions (everybody else deleted automatically)

results2 <- cpAlgorithm(W, k = 3, method = "weighted", I=.14) # obtain final Clique Percolation solution to plot

pdf("Network4.pdf", width=8.5, height=5)
g4 <- cpColoredGraph(W, list.of.communities = results2$list.of.communities.numbers, layout=L, theme='colorblind',
                     labels=names, vsize=6, cut=0, border.width=1.5, 
                     border.color='black', nodeNames = longnames,legend.cex=.37,
                     edge.width = 1, title ="Clique Percolation (optimizing entropy)")
dev.off()

